//serial port routines

void setup_serial(void)
{
	/* relates crystal freq to baud rate - see above and PIC16C7X data sheet p103

	BRGH=1, Fosc=16MHz
	----------------------	
	Baud		SPBRG	
	1200		-
	2400		-
	4800		-
	9600		103
	19200		51
	38400		25
	57600		16
	115200	8

	*/

	SPBRG=103;		// data rate for sending/receiving @ 9600 baud
	BRGH=1;				// data rate for sending/receiving
	SYNC=0;				// asynchronous
	SPEN=1;				// enable serial port pins
	CREN=1;				// enable reception
	SREN=0;				// no effect
	TXIE=0;				// disable tx interrupts
	RCIE=0;				// disable rx interrupts
	TX9=0;				// 8-bit transmission
	RX9=0;				// 8-bit reception
	TXEN=1;				// enable the transmitter
}

//puts a char to serial port
//	this copy of the function is for main()

void putch(unsigned char c)
{
	unsigned char dummy;
	
	while(!TXIF)	// set when register is empty
	{
		if (OERR)
		{
			CREN=0;
			CREN=1;
		}
		if (FERR)
		{
			dummy=RCREG;
		}
	}
	TXREG=c;
}

//puts a char to serial port
//	this copy of the function is for interrupt()

void putch_interrupt(unsigned char c)
{
	unsigned char dummy;
	
	while(!TXIF)	// set when register is empty
	{
		if (OERR)
		{
			CREN=0;
			CREN=1;
		}
		if (FERR)
		{
			dummy=RCREG;
		}
	}
	TXREG=c;
}

//returns whether a character is available in the serial buffer

unsigned char getch_free(void)
{
	unsigned char dummy;
	
	//must software clear overrun error or else all reception dies if 2-byte buffer overruns
	if (OERR)
	{
		CREN=0;
		CREN=1;
	}
	if (FERR)
	{
		dummy=RCREG;
	}
	return RCIF;
}

//gets a character from the serial port without timeout
//	this copy of the function is for main()

unsigned char getch(void)
{
	unsigned char dummy;
	
	while(!RCIF)
	{
		//must software clear overrun error or else all reception dies if 2-byte buffer overruns
		if (OERR)
		{
			CREN=0;
			CREN=1;
		}
		if (FERR)
		{
			dummy=RCREG;
		}
	}
	return RCREG;
}

//gets a character from the serial port without timeout
//	this copy of the function is for interrupt()

unsigned char getch_interrupt(void)
{
	unsigned char dummy;
	
	while(!RCIF)
	{
		//must software clear overrun error or else all reception dies if 2-byte buffer overruns
		if (OERR)
		{
			CREN=0;
			CREN=1;
		}
		if (FERR)
		{
			dummy=RCREG;
		}
	}
	return RCREG;
}



//gets a character from the serial port with timeout, returns 0xFF if error

unsigned char getch_timeout(void)
{
	unsigned char dummy;

	unsigned int timeout_int;

	//typically it takes 833us @ 9600 baud for one character
	timeout_int=timeout_int_us(100100);

	while(timeout_int-- >= 4)
	{
		//must software clear overrun error or else all reception dies if 2-byte buffer overruns
		if (OERR)
		{
			CREN=0;
			CREN=1;
		}
		if (FERR)
		{
			dummy=RCREG;
		}
		if (RCIF) break;		//received character
	}

	if (timeout_int<=2)
	{
		return 0xFF;
	}
	else
	{
		return RCREG;
	}
}

